#! /bin/bash

# TODO: currently setup.sh assumes you're a sudoer, not root

DEST="/usr/local/bin"

# TODO: Make default backend configurable
BACKEND=./backends/git-build

# Autodetect forgehook user if it's already setup
owner=$(find /usr/local/bin/forgehook-backend -maxdepth 0 -printf '%u')
if [[ $? = 0 ]]; then
    echo "[setup.sh] Found existing setup owned by user $owner. Using this user."
else
    # OK it's not setup yet, maybe try a user from argument?
    if [ $# -gt 0 ]; then
        owner="$1"
        echo "[setup.sh] Setup for user $owner"
    else
        # Default value
        owner="forgehook"
        echo "[setup.sh] No setup user found. Using default value $owner"
    fi
fi

# If the user doesn't exist, create it
if ! id -u "$owner" > /dev/null 2>&1; then
    # We need to create the forgehook user
    echo "[setup.sh] User $owner doesn't exist yet. Creating it."
    sudo useradd --create-home --shell /bin/bash --user-group --system --home-dir /opt/forgehook "$owner"
fi


# TODO: Detect if user has UID > 1000 so if it's a real user we refuse to setup
# and try to escalate to generate a dedicated account. 
# Please don't run me as root!
if [[ $EUID = 0 ]]; then
    # TODO: Setup prompt to ask for new user to create and destination folder
    echo "FATAL ERROR: please create a dedicated user to setup forgehook"
    exit 1
fi

# TODO: Check we can escalade privileges

# Everyone can execute (user-facing wrapper)
sudo cp bin/forgehook $DEST
sudo chown "$owner:$owner" $DEST/forgehook

# Only $owner can execute
sudo cp bin/forgehook-backend $DEST
sudo chown "$owner:$owner" $DEST/forgehook-backend
sudo chmod 744 $DEST/forgehook-backend

# Only root should execute
sudo cp bin/forgehook-run $DEST
sudo chown "$owner:$owner" $DEST/forgehook-run
sudo chmod 744 $DEST/forgehook-run

# Everyone can execute (wrapper script for build manager such as git-build)
sudo cp $BACKEND $DEST/forgehook-run-backend
sudo chown "$owner:$owner" $DEST/forgehook-run-backend
sudo chmod 755 $DEST/forgehook-run-backend

echo "[setup.sh] Installed forgehook to $DEST for $owner"

# Auto setup sudo rules
SUDO_SETUP=0
if ! sudo grep "forgehook-backend" /etc/sudoers > /dev/null; then
    SUDO_SETUP=1
    # Need to open a dedicated shell through sudo or we don't have permissions
    sudo sh -c "echo ALL ALL=\("$owner"\) NOPASSWD: /usr/local/bin/forgehook-backend >> /etc/sudoers"
fi
if ! sudo grep "forgehook-run" /etc/sudoers > /dev/null; then
    SUDO_SETUP=1
    sudo sh -c "echo "$owner" ALL=NOPASSWD: /usr/local/bin/forgehook-run >> /etc/sudoers"
fi

[[ $SUDO_SETUP = 1 ]] && echo "[setup.sh] sudo rules have been installed for forgehook"
